package org.eclipse.viatra.cep.core.engine.compiler;

import java.util.Collection;
import java.util.HashSet;
import java.util.Set;
import org.apache.log4j.Logger;
import org.eclipse.emf.common.notify.Notifier;
import org.eclipse.viatra.cep.core.engine.compiler.SameBooleanTransitionTypeMatch;
import org.eclipse.viatra.cep.core.engine.compiler.util.SameBooleanTransitionTypeQuerySpecification;
import org.eclipse.viatra.cep.core.metamodels.automaton.TypedTransition;
import org.eclipse.viatra.query.runtime.api.IMatchProcessor;
import org.eclipse.viatra.query.runtime.api.IQuerySpecification;
import org.eclipse.viatra.query.runtime.api.ViatraQueryEngine;
import org.eclipse.viatra.query.runtime.api.impl.BaseMatcher;
import org.eclipse.viatra.query.runtime.exception.ViatraQueryException;
import org.eclipse.viatra.query.runtime.matchers.tuple.Tuple;
import org.eclipse.viatra.query.runtime.util.ViatraQueryLoggingUtil;

/**
 * Generated pattern matcher API of the org.eclipse.viatra.cep.core.engine.compiler.sameBooleanTransitionType pattern,
 * providing pattern-specific query methods.
 * 
 * <p>Use the pattern matcher on a given model via {@link #on(ViatraQueryEngine)},
 * e.g. in conjunction with {@link ViatraQueryEngine#on(Notifier)}.
 * 
 * <p>Matches of the pattern will be represented as {@link SameBooleanTransitionTypeMatch}.
 * 
 * <p>Original source:
 * <code><pre>
 * pattern sameBooleanTransitionType(transition1 : TypedTransition, transition2 : TypedTransition){
 * 	find negativeTransition(transition1);
 * 	find negativeTransition(transition2);
 * } or {
 * 	neg find negativeTransition(transition1);
 * 	neg find negativeTransition(transition2);
 * }
 * </pre></code>
 * 
 * @see SameBooleanTransitionTypeMatch
 * @see SameBooleanTransitionTypeProcessor
 * @see SameBooleanTransitionTypeQuerySpecification
 * 
 */
@SuppressWarnings("all")
public class SameBooleanTransitionTypeMatcher extends BaseMatcher<SameBooleanTransitionTypeMatch> {
  /**
   * Initializes the pattern matcher within an existing VIATRA Query engine.
   * If the pattern matcher is already constructed in the engine, only a light-weight reference is returned.
   * The match set will be incrementally refreshed upon updates.
   * @param engine the existing VIATRA Query engine in which this matcher will be created.
   * @throws ViatraQueryException if an error occurs during pattern matcher creation
   * 
   */
  public static SameBooleanTransitionTypeMatcher on(final ViatraQueryEngine engine) throws ViatraQueryException {
    // check if matcher already exists
    SameBooleanTransitionTypeMatcher matcher = engine.getExistingMatcher(querySpecification());
    if (matcher == null) {
    	matcher = new SameBooleanTransitionTypeMatcher(engine);
    	// do not have to "put" it into engine.matchers, reportMatcherInitialized() will take care of it
    }
    return matcher;
  }
  
  private final static int POSITION_TRANSITION1 = 0;
  
  private final static int POSITION_TRANSITION2 = 1;
  
  private final static Logger LOGGER = ViatraQueryLoggingUtil.getLogger(SameBooleanTransitionTypeMatcher.class);
  
  /**
   * Initializes the pattern matcher over a given EMF model root (recommended: Resource or ResourceSet).
   * If a pattern matcher is already constructed with the same root, only a light-weight reference is returned.
   * The scope of pattern matching will be the given EMF model root and below (see FAQ for more precise definition).
   * The match set will be incrementally refreshed upon updates from this scope.
   * <p>The matcher will be created within the managed {@link ViatraQueryEngine} belonging to the EMF model root, so
   * multiple matchers will reuse the same engine and benefit from increased performance and reduced memory footprint.
   * @param emfRoot the root of the EMF containment hierarchy where the pattern matcher will operate. Recommended: Resource or ResourceSet.
   * @throws ViatraQueryException if an error occurs during pattern matcher creation
   * @deprecated use {@link #on(ViatraQueryEngine)} instead, e.g. in conjunction with {@link ViatraQueryEngine#on(Notifier)}
   * 
   */
  @Deprecated
  public SameBooleanTransitionTypeMatcher(final Notifier emfRoot) throws ViatraQueryException {
    this(ViatraQueryEngine.on(emfRoot));
  }
  
  /**
   * Initializes the pattern matcher within an existing VIATRA Query engine.
   * If the pattern matcher is already constructed in the engine, only a light-weight reference is returned.
   * The match set will be incrementally refreshed upon updates.
   * @param engine the existing VIATRA Query engine in which this matcher will be created.
   * @throws ViatraQueryException if an error occurs during pattern matcher creation
   * @deprecated use {@link #on(ViatraQueryEngine)} instead
   * 
   */
  @Deprecated
  public SameBooleanTransitionTypeMatcher(final ViatraQueryEngine engine) throws ViatraQueryException {
    super(engine, querySpecification());
  }
  
  /**
   * Returns the set of all matches of the pattern that conform to the given fixed values of some parameters.
   * @param pTransition1 the fixed value of pattern parameter transition1, or null if not bound.
   * @param pTransition2 the fixed value of pattern parameter transition2, or null if not bound.
   * @return matches represented as a SameBooleanTransitionTypeMatch object.
   * 
   */
  public Collection<SameBooleanTransitionTypeMatch> getAllMatches(final TypedTransition pTransition1, final TypedTransition pTransition2) {
    return rawGetAllMatches(new Object[]{pTransition1, pTransition2});
  }
  
  /**
   * Returns an arbitrarily chosen match of the pattern that conforms to the given fixed values of some parameters.
   * Neither determinism nor randomness of selection is guaranteed.
   * @param pTransition1 the fixed value of pattern parameter transition1, or null if not bound.
   * @param pTransition2 the fixed value of pattern parameter transition2, or null if not bound.
   * @return a match represented as a SameBooleanTransitionTypeMatch object, or null if no match is found.
   * 
   */
  public SameBooleanTransitionTypeMatch getOneArbitraryMatch(final TypedTransition pTransition1, final TypedTransition pTransition2) {
    return rawGetOneArbitraryMatch(new Object[]{pTransition1, pTransition2});
  }
  
  /**
   * Indicates whether the given combination of specified pattern parameters constitute a valid pattern match,
   * under any possible substitution of the unspecified parameters (if any).
   * @param pTransition1 the fixed value of pattern parameter transition1, or null if not bound.
   * @param pTransition2 the fixed value of pattern parameter transition2, or null if not bound.
   * @return true if the input is a valid (partial) match of the pattern.
   * 
   */
  public boolean hasMatch(final TypedTransition pTransition1, final TypedTransition pTransition2) {
    return rawHasMatch(new Object[]{pTransition1, pTransition2});
  }
  
  /**
   * Returns the number of all matches of the pattern that conform to the given fixed values of some parameters.
   * @param pTransition1 the fixed value of pattern parameter transition1, or null if not bound.
   * @param pTransition2 the fixed value of pattern parameter transition2, or null if not bound.
   * @return the number of pattern matches found.
   * 
   */
  public int countMatches(final TypedTransition pTransition1, final TypedTransition pTransition2) {
    return rawCountMatches(new Object[]{pTransition1, pTransition2});
  }
  
  /**
   * Executes the given processor on each match of the pattern that conforms to the given fixed values of some parameters.
   * @param pTransition1 the fixed value of pattern parameter transition1, or null if not bound.
   * @param pTransition2 the fixed value of pattern parameter transition2, or null if not bound.
   * @param processor the action that will process each pattern match.
   * 
   */
  public void forEachMatch(final TypedTransition pTransition1, final TypedTransition pTransition2, final IMatchProcessor<? super SameBooleanTransitionTypeMatch> processor) {
    rawForEachMatch(new Object[]{pTransition1, pTransition2}, processor);
  }
  
  /**
   * Executes the given processor on an arbitrarily chosen match of the pattern that conforms to the given fixed values of some parameters.
   * Neither determinism nor randomness of selection is guaranteed.
   * @param pTransition1 the fixed value of pattern parameter transition1, or null if not bound.
   * @param pTransition2 the fixed value of pattern parameter transition2, or null if not bound.
   * @param processor the action that will process the selected match.
   * @return true if the pattern has at least one match with the given parameter values, false if the processor was not invoked
   * 
   */
  public boolean forOneArbitraryMatch(final TypedTransition pTransition1, final TypedTransition pTransition2, final IMatchProcessor<? super SameBooleanTransitionTypeMatch> processor) {
    return rawForOneArbitraryMatch(new Object[]{pTransition1, pTransition2}, processor);
  }
  
  /**
   * Returns a new (partial) match.
   * This can be used e.g. to call the matcher with a partial match.
   * <p>The returned match will be immutable. Use {@link #newEmptyMatch()} to obtain a mutable match object.
   * @param pTransition1 the fixed value of pattern parameter transition1, or null if not bound.
   * @param pTransition2 the fixed value of pattern parameter transition2, or null if not bound.
   * @return the (partial) match object.
   * 
   */
  public SameBooleanTransitionTypeMatch newMatch(final TypedTransition pTransition1, final TypedTransition pTransition2) {
    return SameBooleanTransitionTypeMatch.newMatch(pTransition1, pTransition2);
  }
  
  /**
   * Retrieve the set of values that occur in matches for transition1.
   * @return the Set of all values, null if no parameter with the given name exists, empty set if there are no matches
   * 
   */
  protected Set<TypedTransition> rawAccumulateAllValuesOftransition1(final Object[] parameters) {
    Set<TypedTransition> results = new HashSet<TypedTransition>();
    rawAccumulateAllValues(POSITION_TRANSITION1, parameters, results);
    return results;
  }
  
  /**
   * Retrieve the set of values that occur in matches for transition1.
   * @return the Set of all values, null if no parameter with the given name exists, empty set if there are no matches
   * 
   */
  public Set<TypedTransition> getAllValuesOftransition1() {
    return rawAccumulateAllValuesOftransition1(emptyArray());
  }
  
  /**
   * Retrieve the set of values that occur in matches for transition1.
   * @return the Set of all values, null if no parameter with the given name exists, empty set if there are no matches
   * 
   */
  public Set<TypedTransition> getAllValuesOftransition1(final SameBooleanTransitionTypeMatch partialMatch) {
    return rawAccumulateAllValuesOftransition1(partialMatch.toArray());
  }
  
  /**
   * Retrieve the set of values that occur in matches for transition1.
   * @return the Set of all values, null if no parameter with the given name exists, empty set if there are no matches
   * 
   */
  public Set<TypedTransition> getAllValuesOftransition1(final TypedTransition pTransition2) {
    return rawAccumulateAllValuesOftransition1(new Object[]{
    null, 
    pTransition2
    });
  }
  
  /**
   * Retrieve the set of values that occur in matches for transition2.
   * @return the Set of all values, null if no parameter with the given name exists, empty set if there are no matches
   * 
   */
  protected Set<TypedTransition> rawAccumulateAllValuesOftransition2(final Object[] parameters) {
    Set<TypedTransition> results = new HashSet<TypedTransition>();
    rawAccumulateAllValues(POSITION_TRANSITION2, parameters, results);
    return results;
  }
  
  /**
   * Retrieve the set of values that occur in matches for transition2.
   * @return the Set of all values, null if no parameter with the given name exists, empty set if there are no matches
   * 
   */
  public Set<TypedTransition> getAllValuesOftransition2() {
    return rawAccumulateAllValuesOftransition2(emptyArray());
  }
  
  /**
   * Retrieve the set of values that occur in matches for transition2.
   * @return the Set of all values, null if no parameter with the given name exists, empty set if there are no matches
   * 
   */
  public Set<TypedTransition> getAllValuesOftransition2(final SameBooleanTransitionTypeMatch partialMatch) {
    return rawAccumulateAllValuesOftransition2(partialMatch.toArray());
  }
  
  /**
   * Retrieve the set of values that occur in matches for transition2.
   * @return the Set of all values, null if no parameter with the given name exists, empty set if there are no matches
   * 
   */
  public Set<TypedTransition> getAllValuesOftransition2(final TypedTransition pTransition1) {
    return rawAccumulateAllValuesOftransition2(new Object[]{
    pTransition1, 
    null
    });
  }
  
  @Override
  protected SameBooleanTransitionTypeMatch tupleToMatch(final Tuple t) {
    try {
    	return SameBooleanTransitionTypeMatch.newMatch((TypedTransition) t.get(POSITION_TRANSITION1), (TypedTransition) t.get(POSITION_TRANSITION2));
    } catch(ClassCastException e) {
    	LOGGER.error("Element(s) in tuple not properly typed!",e);
    	return null;
    }
  }
  
  @Override
  protected SameBooleanTransitionTypeMatch arrayToMatch(final Object[] match) {
    try {
    	return SameBooleanTransitionTypeMatch.newMatch((TypedTransition) match[POSITION_TRANSITION1], (TypedTransition) match[POSITION_TRANSITION2]);
    } catch(ClassCastException e) {
    	LOGGER.error("Element(s) in array not properly typed!",e);
    	return null;
    }
  }
  
  @Override
  protected SameBooleanTransitionTypeMatch arrayToMatchMutable(final Object[] match) {
    try {
    	return SameBooleanTransitionTypeMatch.newMutableMatch((TypedTransition) match[POSITION_TRANSITION1], (TypedTransition) match[POSITION_TRANSITION2]);
    } catch(ClassCastException e) {
    	LOGGER.error("Element(s) in array not properly typed!",e);
    	return null;
    }
  }
  
  /**
   * @return the singleton instance of the query specification of this pattern
   * @throws ViatraQueryException if the pattern definition could not be loaded
   * 
   */
  public static IQuerySpecification<SameBooleanTransitionTypeMatcher> querySpecification() throws ViatraQueryException {
    return SameBooleanTransitionTypeQuerySpecification.instance();
  }
}
