package org.eclipse.viatra.cep.core.engine.compiler.util;

import org.eclipse.viatra.cep.core.engine.compiler.EquivalentTransitionsMatch;
import org.eclipse.viatra.cep.core.metamodels.automaton.State;
import org.eclipse.viatra.cep.core.metamodels.automaton.TypedTransition;
import org.eclipse.viatra.query.runtime.api.IMatchProcessor;

/**
 * A match processor tailored for the org.eclipse.viatra.cep.core.engine.compiler.equivalentTransitions pattern.
 * 
 * Clients should derive an (anonymous) class that implements the abstract process().
 * 
 */
@SuppressWarnings("all")
public abstract class EquivalentTransitionsProcessor implements IMatchProcessor<EquivalentTransitionsMatch> {
  /**
   * Defines the action that is to be executed on each match.
   * @param pTransition1 the value of pattern parameter transition1 in the currently processed match
   * @param pTransition2 the value of pattern parameter transition2 in the currently processed match
   * @param pPreState the value of pattern parameter preState in the currently processed match
   * @param pPostState the value of pattern parameter postState in the currently processed match
   * 
   */
  public abstract void process(final TypedTransition pTransition1, final TypedTransition pTransition2, final State pPreState, final State pPostState);
  
  @Override
  public void process(final EquivalentTransitionsMatch match) {
    process(match.getTransition1(), match.getTransition2(), match.getPreState(), match.getPostState());
  }
}
