package org.eclipse.viatra.integration.uml.derivedfeatures;

import java.util.Collection;
import java.util.HashSet;
import java.util.Set;
import org.apache.log4j.Logger;
import org.eclipse.emf.common.notify.Notifier;
import org.eclipse.uml2.uml.Property;
import org.eclipse.viatra.integration.uml.derivedfeatures.PropertyIsCompositeMatch;
import org.eclipse.viatra.integration.uml.derivedfeatures.util.PropertyIsCompositeQuerySpecification;
import org.eclipse.viatra.query.runtime.api.IMatchProcessor;
import org.eclipse.viatra.query.runtime.api.IQuerySpecification;
import org.eclipse.viatra.query.runtime.api.ViatraQueryEngine;
import org.eclipse.viatra.query.runtime.api.impl.BaseMatcher;
import org.eclipse.viatra.query.runtime.exception.ViatraQueryException;
import org.eclipse.viatra.query.runtime.matchers.tuple.Tuple;
import org.eclipse.viatra.query.runtime.util.ViatraQueryLoggingUtil;

/**
 * Generated pattern matcher API of the org.eclipse.viatra.integration.uml.derivedfeatures.propertyIsComposite pattern,
 * providing pattern-specific query methods.
 * 
 * <p>Use the pattern matcher on a given model via {@link #on(ViatraQueryEngine)},
 * e.g. in conjunction with {@link ViatraQueryEngine#on(Notifier)}.
 * 
 * <p>Matches of the pattern will be represented as {@link PropertyIsCompositeMatch}.
 * 
 * <p>Original source:
 * <code><pre>
 * (aggregation = AggregationKind::composite)
 * 
 * {@literal @}Surrogate(feature = "isComposite")
 * {@literal @}QueryExplorer(checked = false)
 * pattern propertyIsComposite(
 *     property : Property, isComposite
 * ) {
 *     Property.aggregation(property, AggregationKind::composite);
 *     isComposite == true;
 * } or {
 *     Property.aggregation(property, aggregation);
 *     aggregation != AggregationKind::composite;
 *     isComposite == false;
 * }
 * </pre></code>
 * 
 * @see PropertyIsCompositeMatch
 * @see PropertyIsCompositeProcessor
 * @see PropertyIsCompositeQuerySpecification
 * 
 */
@SuppressWarnings("all")
public class PropertyIsCompositeMatcher extends BaseMatcher<PropertyIsCompositeMatch> {
  /**
   * Initializes the pattern matcher within an existing VIATRA Query engine.
   * If the pattern matcher is already constructed in the engine, only a light-weight reference is returned.
   * The match set will be incrementally refreshed upon updates.
   * @param engine the existing VIATRA Query engine in which this matcher will be created.
   * @throws ViatraQueryException if an error occurs during pattern matcher creation
   * 
   */
  public static PropertyIsCompositeMatcher on(final ViatraQueryEngine engine) throws ViatraQueryException {
    // check if matcher already exists
    PropertyIsCompositeMatcher matcher = engine.getExistingMatcher(querySpecification());
    if (matcher == null) {
    	matcher = new PropertyIsCompositeMatcher(engine);
    	// do not have to "put" it into engine.matchers, reportMatcherInitialized() will take care of it
    }
    return matcher;
  }
  
  private final static int POSITION_PROPERTY = 0;
  
  private final static int POSITION_ISCOMPOSITE = 1;
  
  private final static Logger LOGGER = ViatraQueryLoggingUtil.getLogger(PropertyIsCompositeMatcher.class);
  
  /**
   * Initializes the pattern matcher over a given EMF model root (recommended: Resource or ResourceSet).
   * If a pattern matcher is already constructed with the same root, only a light-weight reference is returned.
   * The scope of pattern matching will be the given EMF model root and below (see FAQ for more precise definition).
   * The match set will be incrementally refreshed upon updates from this scope.
   * <p>The matcher will be created within the managed {@link ViatraQueryEngine} belonging to the EMF model root, so
   * multiple matchers will reuse the same engine and benefit from increased performance and reduced memory footprint.
   * @param emfRoot the root of the EMF containment hierarchy where the pattern matcher will operate. Recommended: Resource or ResourceSet.
   * @throws ViatraQueryException if an error occurs during pattern matcher creation
   * @deprecated use {@link #on(ViatraQueryEngine)} instead, e.g. in conjunction with {@link ViatraQueryEngine#on(Notifier)}
   * 
   */
  @Deprecated
  public PropertyIsCompositeMatcher(final Notifier emfRoot) throws ViatraQueryException {
    this(ViatraQueryEngine.on(emfRoot));
  }
  
  /**
   * Initializes the pattern matcher within an existing VIATRA Query engine.
   * If the pattern matcher is already constructed in the engine, only a light-weight reference is returned.
   * The match set will be incrementally refreshed upon updates.
   * @param engine the existing VIATRA Query engine in which this matcher will be created.
   * @throws ViatraQueryException if an error occurs during pattern matcher creation
   * @deprecated use {@link #on(ViatraQueryEngine)} instead
   * 
   */
  @Deprecated
  public PropertyIsCompositeMatcher(final ViatraQueryEngine engine) throws ViatraQueryException {
    super(engine, querySpecification());
  }
  
  /**
   * Returns the set of all matches of the pattern that conform to the given fixed values of some parameters.
   * @param pProperty the fixed value of pattern parameter property, or null if not bound.
   * @param pIsComposite the fixed value of pattern parameter isComposite, or null if not bound.
   * @return matches represented as a PropertyIsCompositeMatch object.
   * 
   */
  public Collection<PropertyIsCompositeMatch> getAllMatches(final Property pProperty, final Boolean pIsComposite) {
    return rawGetAllMatches(new Object[]{pProperty, pIsComposite});
  }
  
  /**
   * Returns an arbitrarily chosen match of the pattern that conforms to the given fixed values of some parameters.
   * Neither determinism nor randomness of selection is guaranteed.
   * @param pProperty the fixed value of pattern parameter property, or null if not bound.
   * @param pIsComposite the fixed value of pattern parameter isComposite, or null if not bound.
   * @return a match represented as a PropertyIsCompositeMatch object, or null if no match is found.
   * 
   */
  public PropertyIsCompositeMatch getOneArbitraryMatch(final Property pProperty, final Boolean pIsComposite) {
    return rawGetOneArbitraryMatch(new Object[]{pProperty, pIsComposite});
  }
  
  /**
   * Indicates whether the given combination of specified pattern parameters constitute a valid pattern match,
   * under any possible substitution of the unspecified parameters (if any).
   * @param pProperty the fixed value of pattern parameter property, or null if not bound.
   * @param pIsComposite the fixed value of pattern parameter isComposite, or null if not bound.
   * @return true if the input is a valid (partial) match of the pattern.
   * 
   */
  public boolean hasMatch(final Property pProperty, final Boolean pIsComposite) {
    return rawHasMatch(new Object[]{pProperty, pIsComposite});
  }
  
  /**
   * Returns the number of all matches of the pattern that conform to the given fixed values of some parameters.
   * @param pProperty the fixed value of pattern parameter property, or null if not bound.
   * @param pIsComposite the fixed value of pattern parameter isComposite, or null if not bound.
   * @return the number of pattern matches found.
   * 
   */
  public int countMatches(final Property pProperty, final Boolean pIsComposite) {
    return rawCountMatches(new Object[]{pProperty, pIsComposite});
  }
  
  /**
   * Executes the given processor on each match of the pattern that conforms to the given fixed values of some parameters.
   * @param pProperty the fixed value of pattern parameter property, or null if not bound.
   * @param pIsComposite the fixed value of pattern parameter isComposite, or null if not bound.
   * @param processor the action that will process each pattern match.
   * 
   */
  public void forEachMatch(final Property pProperty, final Boolean pIsComposite, final IMatchProcessor<? super PropertyIsCompositeMatch> processor) {
    rawForEachMatch(new Object[]{pProperty, pIsComposite}, processor);
  }
  
  /**
   * Executes the given processor on an arbitrarily chosen match of the pattern that conforms to the given fixed values of some parameters.
   * Neither determinism nor randomness of selection is guaranteed.
   * @param pProperty the fixed value of pattern parameter property, or null if not bound.
   * @param pIsComposite the fixed value of pattern parameter isComposite, or null if not bound.
   * @param processor the action that will process the selected match.
   * @return true if the pattern has at least one match with the given parameter values, false if the processor was not invoked
   * 
   */
  public boolean forOneArbitraryMatch(final Property pProperty, final Boolean pIsComposite, final IMatchProcessor<? super PropertyIsCompositeMatch> processor) {
    return rawForOneArbitraryMatch(new Object[]{pProperty, pIsComposite}, processor);
  }
  
  /**
   * Returns a new (partial) match.
   * This can be used e.g. to call the matcher with a partial match.
   * <p>The returned match will be immutable. Use {@link #newEmptyMatch()} to obtain a mutable match object.
   * @param pProperty the fixed value of pattern parameter property, or null if not bound.
   * @param pIsComposite the fixed value of pattern parameter isComposite, or null if not bound.
   * @return the (partial) match object.
   * 
   */
  public PropertyIsCompositeMatch newMatch(final Property pProperty, final Boolean pIsComposite) {
    return PropertyIsCompositeMatch.newMatch(pProperty, pIsComposite);
  }
  
  /**
   * Retrieve the set of values that occur in matches for property.
   * @return the Set of all values, null if no parameter with the given name exists, empty set if there are no matches
   * 
   */
  protected Set<Property> rawAccumulateAllValuesOfproperty(final Object[] parameters) {
    Set<Property> results = new HashSet<Property>();
    rawAccumulateAllValues(POSITION_PROPERTY, parameters, results);
    return results;
  }
  
  /**
   * Retrieve the set of values that occur in matches for property.
   * @return the Set of all values, null if no parameter with the given name exists, empty set if there are no matches
   * 
   */
  public Set<Property> getAllValuesOfproperty() {
    return rawAccumulateAllValuesOfproperty(emptyArray());
  }
  
  /**
   * Retrieve the set of values that occur in matches for property.
   * @return the Set of all values, null if no parameter with the given name exists, empty set if there are no matches
   * 
   */
  public Set<Property> getAllValuesOfproperty(final PropertyIsCompositeMatch partialMatch) {
    return rawAccumulateAllValuesOfproperty(partialMatch.toArray());
  }
  
  /**
   * Retrieve the set of values that occur in matches for property.
   * @return the Set of all values, null if no parameter with the given name exists, empty set if there are no matches
   * 
   */
  public Set<Property> getAllValuesOfproperty(final Boolean pIsComposite) {
    return rawAccumulateAllValuesOfproperty(new Object[]{
    null, 
    pIsComposite
    });
  }
  
  /**
   * Retrieve the set of values that occur in matches for isComposite.
   * @return the Set of all values, null if no parameter with the given name exists, empty set if there are no matches
   * 
   */
  protected Set<Boolean> rawAccumulateAllValuesOfisComposite(final Object[] parameters) {
    Set<Boolean> results = new HashSet<Boolean>();
    rawAccumulateAllValues(POSITION_ISCOMPOSITE, parameters, results);
    return results;
  }
  
  /**
   * Retrieve the set of values that occur in matches for isComposite.
   * @return the Set of all values, null if no parameter with the given name exists, empty set if there are no matches
   * 
   */
  public Set<Boolean> getAllValuesOfisComposite() {
    return rawAccumulateAllValuesOfisComposite(emptyArray());
  }
  
  /**
   * Retrieve the set of values that occur in matches for isComposite.
   * @return the Set of all values, null if no parameter with the given name exists, empty set if there are no matches
   * 
   */
  public Set<Boolean> getAllValuesOfisComposite(final PropertyIsCompositeMatch partialMatch) {
    return rawAccumulateAllValuesOfisComposite(partialMatch.toArray());
  }
  
  /**
   * Retrieve the set of values that occur in matches for isComposite.
   * @return the Set of all values, null if no parameter with the given name exists, empty set if there are no matches
   * 
   */
  public Set<Boolean> getAllValuesOfisComposite(final Property pProperty) {
    return rawAccumulateAllValuesOfisComposite(new Object[]{
    pProperty, 
    null
    });
  }
  
  @Override
  protected PropertyIsCompositeMatch tupleToMatch(final Tuple t) {
    try {
    	return PropertyIsCompositeMatch.newMatch((Property) t.get(POSITION_PROPERTY), (Boolean) t.get(POSITION_ISCOMPOSITE));
    } catch(ClassCastException e) {
    	LOGGER.error("Element(s) in tuple not properly typed!",e);
    	return null;
    }
  }
  
  @Override
  protected PropertyIsCompositeMatch arrayToMatch(final Object[] match) {
    try {
    	return PropertyIsCompositeMatch.newMatch((Property) match[POSITION_PROPERTY], (Boolean) match[POSITION_ISCOMPOSITE]);
    } catch(ClassCastException e) {
    	LOGGER.error("Element(s) in array not properly typed!",e);
    	return null;
    }
  }
  
  @Override
  protected PropertyIsCompositeMatch arrayToMatchMutable(final Object[] match) {
    try {
    	return PropertyIsCompositeMatch.newMutableMatch((Property) match[POSITION_PROPERTY], (Boolean) match[POSITION_ISCOMPOSITE]);
    } catch(ClassCastException e) {
    	LOGGER.error("Element(s) in array not properly typed!",e);
    	return null;
    }
  }
  
  /**
   * @return the singleton instance of the query specification of this pattern
   * @throws ViatraQueryException if the pattern definition could not be loaded
   * 
   */
  public static IQuerySpecification<PropertyIsCompositeMatcher> querySpecification() throws ViatraQueryException {
    return PropertyIsCompositeQuerySpecification.instance();
  }
}
