package org.eclipse.viatra.integration.uml.derivedfeatures;

import java.util.Collection;
import java.util.HashSet;
import java.util.Set;
import org.apache.log4j.Logger;
import org.eclipse.emf.common.notify.Notifier;
import org.eclipse.uml2.uml.State;
import org.eclipse.viatra.integration.uml.derivedfeatures.StateIsCompositeMatch;
import org.eclipse.viatra.integration.uml.derivedfeatures.util.StateIsCompositeQuerySpecification;
import org.eclipse.viatra.query.runtime.api.IMatchProcessor;
import org.eclipse.viatra.query.runtime.api.IQuerySpecification;
import org.eclipse.viatra.query.runtime.api.ViatraQueryEngine;
import org.eclipse.viatra.query.runtime.api.impl.BaseMatcher;
import org.eclipse.viatra.query.runtime.exception.ViatraQueryException;
import org.eclipse.viatra.query.runtime.matchers.tuple.Tuple;
import org.eclipse.viatra.query.runtime.util.ViatraQueryLoggingUtil;

/**
 * Generated pattern matcher API of the org.eclipse.viatra.integration.uml.derivedfeatures.stateIsComposite pattern,
 * providing pattern-specific query methods.
 * 
 * <p>Use the pattern matcher on a given model via {@link #on(ViatraQueryEngine)},
 * e.g. in conjunction with {@link ViatraQueryEngine#on(Notifier)}.
 * 
 * <p>Matches of the pattern will be represented as {@link StateIsCompositeMatch}.
 * 
 * <p>Original source:
 * <code><pre>
 * (region-{@literal >}notEmpty())
 * 
 * {@literal @}Surrogate(feature = "isComposite")
 * {@literal @}QueryExplorer(checked = false)
 * pattern stateIsComposite(
 *     self : State, isComposite
 * ) {
 *     find stateIsComposite_0(self, _region);
 *     isComposite == true;
 * } or {
 *     neg find stateIsComposite_0(self, _region);
 *     isComposite == false;
 * }
 * </pre></code>
 * 
 * @see StateIsCompositeMatch
 * @see StateIsCompositeProcessor
 * @see StateIsCompositeQuerySpecification
 * 
 */
@SuppressWarnings("all")
public class StateIsCompositeMatcher extends BaseMatcher<StateIsCompositeMatch> {
  /**
   * Initializes the pattern matcher within an existing VIATRA Query engine.
   * If the pattern matcher is already constructed in the engine, only a light-weight reference is returned.
   * The match set will be incrementally refreshed upon updates.
   * @param engine the existing VIATRA Query engine in which this matcher will be created.
   * @throws ViatraQueryException if an error occurs during pattern matcher creation
   * 
   */
  public static StateIsCompositeMatcher on(final ViatraQueryEngine engine) throws ViatraQueryException {
    // check if matcher already exists
    StateIsCompositeMatcher matcher = engine.getExistingMatcher(querySpecification());
    if (matcher == null) {
    	matcher = new StateIsCompositeMatcher(engine);
    	// do not have to "put" it into engine.matchers, reportMatcherInitialized() will take care of it
    }
    return matcher;
  }
  
  private final static int POSITION_SELF = 0;
  
  private final static int POSITION_ISCOMPOSITE = 1;
  
  private final static Logger LOGGER = ViatraQueryLoggingUtil.getLogger(StateIsCompositeMatcher.class);
  
  /**
   * Initializes the pattern matcher over a given EMF model root (recommended: Resource or ResourceSet).
   * If a pattern matcher is already constructed with the same root, only a light-weight reference is returned.
   * The scope of pattern matching will be the given EMF model root and below (see FAQ for more precise definition).
   * The match set will be incrementally refreshed upon updates from this scope.
   * <p>The matcher will be created within the managed {@link ViatraQueryEngine} belonging to the EMF model root, so
   * multiple matchers will reuse the same engine and benefit from increased performance and reduced memory footprint.
   * @param emfRoot the root of the EMF containment hierarchy where the pattern matcher will operate. Recommended: Resource or ResourceSet.
   * @throws ViatraQueryException if an error occurs during pattern matcher creation
   * @deprecated use {@link #on(ViatraQueryEngine)} instead, e.g. in conjunction with {@link ViatraQueryEngine#on(Notifier)}
   * 
   */
  @Deprecated
  public StateIsCompositeMatcher(final Notifier emfRoot) throws ViatraQueryException {
    this(ViatraQueryEngine.on(emfRoot));
  }
  
  /**
   * Initializes the pattern matcher within an existing VIATRA Query engine.
   * If the pattern matcher is already constructed in the engine, only a light-weight reference is returned.
   * The match set will be incrementally refreshed upon updates.
   * @param engine the existing VIATRA Query engine in which this matcher will be created.
   * @throws ViatraQueryException if an error occurs during pattern matcher creation
   * @deprecated use {@link #on(ViatraQueryEngine)} instead
   * 
   */
  @Deprecated
  public StateIsCompositeMatcher(final ViatraQueryEngine engine) throws ViatraQueryException {
    super(engine, querySpecification());
  }
  
  /**
   * Returns the set of all matches of the pattern that conform to the given fixed values of some parameters.
   * @param pSelf the fixed value of pattern parameter self, or null if not bound.
   * @param pIsComposite the fixed value of pattern parameter isComposite, or null if not bound.
   * @return matches represented as a StateIsCompositeMatch object.
   * 
   */
  public Collection<StateIsCompositeMatch> getAllMatches(final State pSelf, final Boolean pIsComposite) {
    return rawGetAllMatches(new Object[]{pSelf, pIsComposite});
  }
  
  /**
   * Returns an arbitrarily chosen match of the pattern that conforms to the given fixed values of some parameters.
   * Neither determinism nor randomness of selection is guaranteed.
   * @param pSelf the fixed value of pattern parameter self, or null if not bound.
   * @param pIsComposite the fixed value of pattern parameter isComposite, or null if not bound.
   * @return a match represented as a StateIsCompositeMatch object, or null if no match is found.
   * 
   */
  public StateIsCompositeMatch getOneArbitraryMatch(final State pSelf, final Boolean pIsComposite) {
    return rawGetOneArbitraryMatch(new Object[]{pSelf, pIsComposite});
  }
  
  /**
   * Indicates whether the given combination of specified pattern parameters constitute a valid pattern match,
   * under any possible substitution of the unspecified parameters (if any).
   * @param pSelf the fixed value of pattern parameter self, or null if not bound.
   * @param pIsComposite the fixed value of pattern parameter isComposite, or null if not bound.
   * @return true if the input is a valid (partial) match of the pattern.
   * 
   */
  public boolean hasMatch(final State pSelf, final Boolean pIsComposite) {
    return rawHasMatch(new Object[]{pSelf, pIsComposite});
  }
  
  /**
   * Returns the number of all matches of the pattern that conform to the given fixed values of some parameters.
   * @param pSelf the fixed value of pattern parameter self, or null if not bound.
   * @param pIsComposite the fixed value of pattern parameter isComposite, or null if not bound.
   * @return the number of pattern matches found.
   * 
   */
  public int countMatches(final State pSelf, final Boolean pIsComposite) {
    return rawCountMatches(new Object[]{pSelf, pIsComposite});
  }
  
  /**
   * Executes the given processor on each match of the pattern that conforms to the given fixed values of some parameters.
   * @param pSelf the fixed value of pattern parameter self, or null if not bound.
   * @param pIsComposite the fixed value of pattern parameter isComposite, or null if not bound.
   * @param processor the action that will process each pattern match.
   * 
   */
  public void forEachMatch(final State pSelf, final Boolean pIsComposite, final IMatchProcessor<? super StateIsCompositeMatch> processor) {
    rawForEachMatch(new Object[]{pSelf, pIsComposite}, processor);
  }
  
  /**
   * Executes the given processor on an arbitrarily chosen match of the pattern that conforms to the given fixed values of some parameters.
   * Neither determinism nor randomness of selection is guaranteed.
   * @param pSelf the fixed value of pattern parameter self, or null if not bound.
   * @param pIsComposite the fixed value of pattern parameter isComposite, or null if not bound.
   * @param processor the action that will process the selected match.
   * @return true if the pattern has at least one match with the given parameter values, false if the processor was not invoked
   * 
   */
  public boolean forOneArbitraryMatch(final State pSelf, final Boolean pIsComposite, final IMatchProcessor<? super StateIsCompositeMatch> processor) {
    return rawForOneArbitraryMatch(new Object[]{pSelf, pIsComposite}, processor);
  }
  
  /**
   * Returns a new (partial) match.
   * This can be used e.g. to call the matcher with a partial match.
   * <p>The returned match will be immutable. Use {@link #newEmptyMatch()} to obtain a mutable match object.
   * @param pSelf the fixed value of pattern parameter self, or null if not bound.
   * @param pIsComposite the fixed value of pattern parameter isComposite, or null if not bound.
   * @return the (partial) match object.
   * 
   */
  public StateIsCompositeMatch newMatch(final State pSelf, final Boolean pIsComposite) {
    return StateIsCompositeMatch.newMatch(pSelf, pIsComposite);
  }
  
  /**
   * Retrieve the set of values that occur in matches for self.
   * @return the Set of all values, null if no parameter with the given name exists, empty set if there are no matches
   * 
   */
  protected Set<State> rawAccumulateAllValuesOfself(final Object[] parameters) {
    Set<State> results = new HashSet<State>();
    rawAccumulateAllValues(POSITION_SELF, parameters, results);
    return results;
  }
  
  /**
   * Retrieve the set of values that occur in matches for self.
   * @return the Set of all values, null if no parameter with the given name exists, empty set if there are no matches
   * 
   */
  public Set<State> getAllValuesOfself() {
    return rawAccumulateAllValuesOfself(emptyArray());
  }
  
  /**
   * Retrieve the set of values that occur in matches for self.
   * @return the Set of all values, null if no parameter with the given name exists, empty set if there are no matches
   * 
   */
  public Set<State> getAllValuesOfself(final StateIsCompositeMatch partialMatch) {
    return rawAccumulateAllValuesOfself(partialMatch.toArray());
  }
  
  /**
   * Retrieve the set of values that occur in matches for self.
   * @return the Set of all values, null if no parameter with the given name exists, empty set if there are no matches
   * 
   */
  public Set<State> getAllValuesOfself(final Boolean pIsComposite) {
    return rawAccumulateAllValuesOfself(new Object[]{
    null, 
    pIsComposite
    });
  }
  
  /**
   * Retrieve the set of values that occur in matches for isComposite.
   * @return the Set of all values, null if no parameter with the given name exists, empty set if there are no matches
   * 
   */
  protected Set<Boolean> rawAccumulateAllValuesOfisComposite(final Object[] parameters) {
    Set<Boolean> results = new HashSet<Boolean>();
    rawAccumulateAllValues(POSITION_ISCOMPOSITE, parameters, results);
    return results;
  }
  
  /**
   * Retrieve the set of values that occur in matches for isComposite.
   * @return the Set of all values, null if no parameter with the given name exists, empty set if there are no matches
   * 
   */
  public Set<Boolean> getAllValuesOfisComposite() {
    return rawAccumulateAllValuesOfisComposite(emptyArray());
  }
  
  /**
   * Retrieve the set of values that occur in matches for isComposite.
   * @return the Set of all values, null if no parameter with the given name exists, empty set if there are no matches
   * 
   */
  public Set<Boolean> getAllValuesOfisComposite(final StateIsCompositeMatch partialMatch) {
    return rawAccumulateAllValuesOfisComposite(partialMatch.toArray());
  }
  
  /**
   * Retrieve the set of values that occur in matches for isComposite.
   * @return the Set of all values, null if no parameter with the given name exists, empty set if there are no matches
   * 
   */
  public Set<Boolean> getAllValuesOfisComposite(final State pSelf) {
    return rawAccumulateAllValuesOfisComposite(new Object[]{
    pSelf, 
    null
    });
  }
  
  @Override
  protected StateIsCompositeMatch tupleToMatch(final Tuple t) {
    try {
    	return StateIsCompositeMatch.newMatch((State) t.get(POSITION_SELF), (Boolean) t.get(POSITION_ISCOMPOSITE));
    } catch(ClassCastException e) {
    	LOGGER.error("Element(s) in tuple not properly typed!",e);
    	return null;
    }
  }
  
  @Override
  protected StateIsCompositeMatch arrayToMatch(final Object[] match) {
    try {
    	return StateIsCompositeMatch.newMatch((State) match[POSITION_SELF], (Boolean) match[POSITION_ISCOMPOSITE]);
    } catch(ClassCastException e) {
    	LOGGER.error("Element(s) in array not properly typed!",e);
    	return null;
    }
  }
  
  @Override
  protected StateIsCompositeMatch arrayToMatchMutable(final Object[] match) {
    try {
    	return StateIsCompositeMatch.newMutableMatch((State) match[POSITION_SELF], (Boolean) match[POSITION_ISCOMPOSITE]);
    } catch(ClassCastException e) {
    	LOGGER.error("Element(s) in array not properly typed!",e);
    	return null;
    }
  }
  
  /**
   * @return the singleton instance of the query specification of this pattern
   * @throws ViatraQueryException if the pattern definition could not be loaded
   * 
   */
  public static IQuerySpecification<StateIsCompositeMatcher> querySpecification() throws ViatraQueryException {
    return StateIsCompositeQuerySpecification.instance();
  }
}
