package org.eclipse.viatra.query.tooling.ui.retevis;

import java.util.Collection;
import java.util.HashSet;
import java.util.Set;
import org.apache.log4j.Logger;
import org.eclipse.emf.common.notify.Notifier;
import org.eclipse.viatra.query.runtime.api.IMatchProcessor;
import org.eclipse.viatra.query.runtime.api.IQuerySpecification;
import org.eclipse.viatra.query.runtime.api.ViatraQueryEngine;
import org.eclipse.viatra.query.runtime.api.impl.BaseMatcher;
import org.eclipse.viatra.query.runtime.exception.ViatraQueryException;
import org.eclipse.viatra.query.runtime.matchers.tuple.Tuple;
import org.eclipse.viatra.query.runtime.rete.recipes.BetaRecipe;
import org.eclipse.viatra.query.runtime.rete.recipes.ProjectionIndexerRecipe;
import org.eclipse.viatra.query.runtime.util.ViatraQueryLoggingUtil;
import org.eclipse.viatra.query.tooling.ui.retevis.BetaRecipeLeftParentMatch;
import org.eclipse.viatra.query.tooling.ui.retevis.util.BetaRecipeLeftParentQuerySpecification;

/**
 * Generated pattern matcher API of the org.eclipse.viatra.query.tooling.ui.retevis.betaRecipeLeftParent pattern,
 * providing pattern-specific query methods.
 * 
 * <p>Use the pattern matcher on a given model via {@link #on(ViatraQueryEngine)},
 * e.g. in conjunction with {@link ViatraQueryEngine#on(Notifier)}.
 * 
 * <p>Matches of the pattern will be represented as {@link BetaRecipeLeftParentMatch}.
 * 
 * <p>Original source:
 * <code><pre>
 * {@literal @}Edge(source = parent, target = recipe, label = "left")
 * {@literal @}QueryExplorer(checked = false)
 * pattern betaRecipeLeftParent(recipe: BetaRecipe, parent: ProjectionIndexerRecipe) {
 * 	BetaRecipe.leftParent(recipe, parent);	
 * }
 * </pre></code>
 * 
 * @see BetaRecipeLeftParentMatch
 * @see BetaRecipeLeftParentProcessor
 * @see BetaRecipeLeftParentQuerySpecification
 * 
 */
@SuppressWarnings("all")
public class BetaRecipeLeftParentMatcher extends BaseMatcher<BetaRecipeLeftParentMatch> {
  /**
   * Initializes the pattern matcher within an existing VIATRA Query engine.
   * If the pattern matcher is already constructed in the engine, only a light-weight reference is returned.
   * The match set will be incrementally refreshed upon updates.
   * @param engine the existing VIATRA Query engine in which this matcher will be created.
   * @throws ViatraQueryException if an error occurs during pattern matcher creation
   * 
   */
  public static BetaRecipeLeftParentMatcher on(final ViatraQueryEngine engine) throws ViatraQueryException {
    // check if matcher already exists
    BetaRecipeLeftParentMatcher matcher = engine.getExistingMatcher(querySpecification());
    if (matcher == null) {
    	matcher = new BetaRecipeLeftParentMatcher(engine);
    	// do not have to "put" it into engine.matchers, reportMatcherInitialized() will take care of it
    }
    return matcher;
  }
  
  private final static int POSITION_RECIPE = 0;
  
  private final static int POSITION_PARENT = 1;
  
  private final static Logger LOGGER = ViatraQueryLoggingUtil.getLogger(BetaRecipeLeftParentMatcher.class);
  
  /**
   * Initializes the pattern matcher over a given EMF model root (recommended: Resource or ResourceSet).
   * If a pattern matcher is already constructed with the same root, only a light-weight reference is returned.
   * The scope of pattern matching will be the given EMF model root and below (see FAQ for more precise definition).
   * The match set will be incrementally refreshed upon updates from this scope.
   * <p>The matcher will be created within the managed {@link ViatraQueryEngine} belonging to the EMF model root, so
   * multiple matchers will reuse the same engine and benefit from increased performance and reduced memory footprint.
   * @param emfRoot the root of the EMF containment hierarchy where the pattern matcher will operate. Recommended: Resource or ResourceSet.
   * @throws ViatraQueryException if an error occurs during pattern matcher creation
   * @deprecated use {@link #on(ViatraQueryEngine)} instead, e.g. in conjunction with {@link ViatraQueryEngine#on(Notifier)}
   * 
   */
  @Deprecated
  public BetaRecipeLeftParentMatcher(final Notifier emfRoot) throws ViatraQueryException {
    this(ViatraQueryEngine.on(emfRoot));
  }
  
  /**
   * Initializes the pattern matcher within an existing VIATRA Query engine.
   * If the pattern matcher is already constructed in the engine, only a light-weight reference is returned.
   * The match set will be incrementally refreshed upon updates.
   * @param engine the existing VIATRA Query engine in which this matcher will be created.
   * @throws ViatraQueryException if an error occurs during pattern matcher creation
   * @deprecated use {@link #on(ViatraQueryEngine)} instead
   * 
   */
  @Deprecated
  public BetaRecipeLeftParentMatcher(final ViatraQueryEngine engine) throws ViatraQueryException {
    super(engine, querySpecification());
  }
  
  /**
   * Returns the set of all matches of the pattern that conform to the given fixed values of some parameters.
   * @param pRecipe the fixed value of pattern parameter recipe, or null if not bound.
   * @param pParent the fixed value of pattern parameter parent, or null if not bound.
   * @return matches represented as a BetaRecipeLeftParentMatch object.
   * 
   */
  public Collection<BetaRecipeLeftParentMatch> getAllMatches(final BetaRecipe pRecipe, final ProjectionIndexerRecipe pParent) {
    return rawGetAllMatches(new Object[]{pRecipe, pParent});
  }
  
  /**
   * Returns an arbitrarily chosen match of the pattern that conforms to the given fixed values of some parameters.
   * Neither determinism nor randomness of selection is guaranteed.
   * @param pRecipe the fixed value of pattern parameter recipe, or null if not bound.
   * @param pParent the fixed value of pattern parameter parent, or null if not bound.
   * @return a match represented as a BetaRecipeLeftParentMatch object, or null if no match is found.
   * 
   */
  public BetaRecipeLeftParentMatch getOneArbitraryMatch(final BetaRecipe pRecipe, final ProjectionIndexerRecipe pParent) {
    return rawGetOneArbitraryMatch(new Object[]{pRecipe, pParent});
  }
  
  /**
   * Indicates whether the given combination of specified pattern parameters constitute a valid pattern match,
   * under any possible substitution of the unspecified parameters (if any).
   * @param pRecipe the fixed value of pattern parameter recipe, or null if not bound.
   * @param pParent the fixed value of pattern parameter parent, or null if not bound.
   * @return true if the input is a valid (partial) match of the pattern.
   * 
   */
  public boolean hasMatch(final BetaRecipe pRecipe, final ProjectionIndexerRecipe pParent) {
    return rawHasMatch(new Object[]{pRecipe, pParent});
  }
  
  /**
   * Returns the number of all matches of the pattern that conform to the given fixed values of some parameters.
   * @param pRecipe the fixed value of pattern parameter recipe, or null if not bound.
   * @param pParent the fixed value of pattern parameter parent, or null if not bound.
   * @return the number of pattern matches found.
   * 
   */
  public int countMatches(final BetaRecipe pRecipe, final ProjectionIndexerRecipe pParent) {
    return rawCountMatches(new Object[]{pRecipe, pParent});
  }
  
  /**
   * Executes the given processor on each match of the pattern that conforms to the given fixed values of some parameters.
   * @param pRecipe the fixed value of pattern parameter recipe, or null if not bound.
   * @param pParent the fixed value of pattern parameter parent, or null if not bound.
   * @param processor the action that will process each pattern match.
   * 
   */
  public void forEachMatch(final BetaRecipe pRecipe, final ProjectionIndexerRecipe pParent, final IMatchProcessor<? super BetaRecipeLeftParentMatch> processor) {
    rawForEachMatch(new Object[]{pRecipe, pParent}, processor);
  }
  
  /**
   * Executes the given processor on an arbitrarily chosen match of the pattern that conforms to the given fixed values of some parameters.
   * Neither determinism nor randomness of selection is guaranteed.
   * @param pRecipe the fixed value of pattern parameter recipe, or null if not bound.
   * @param pParent the fixed value of pattern parameter parent, or null if not bound.
   * @param processor the action that will process the selected match.
   * @return true if the pattern has at least one match with the given parameter values, false if the processor was not invoked
   * 
   */
  public boolean forOneArbitraryMatch(final BetaRecipe pRecipe, final ProjectionIndexerRecipe pParent, final IMatchProcessor<? super BetaRecipeLeftParentMatch> processor) {
    return rawForOneArbitraryMatch(new Object[]{pRecipe, pParent}, processor);
  }
  
  /**
   * Returns a new (partial) match.
   * This can be used e.g. to call the matcher with a partial match.
   * <p>The returned match will be immutable. Use {@link #newEmptyMatch()} to obtain a mutable match object.
   * @param pRecipe the fixed value of pattern parameter recipe, or null if not bound.
   * @param pParent the fixed value of pattern parameter parent, or null if not bound.
   * @return the (partial) match object.
   * 
   */
  public BetaRecipeLeftParentMatch newMatch(final BetaRecipe pRecipe, final ProjectionIndexerRecipe pParent) {
    return BetaRecipeLeftParentMatch.newMatch(pRecipe, pParent);
  }
  
  /**
   * Retrieve the set of values that occur in matches for recipe.
   * @return the Set of all values, null if no parameter with the given name exists, empty set if there are no matches
   * 
   */
  protected Set<BetaRecipe> rawAccumulateAllValuesOfrecipe(final Object[] parameters) {
    Set<BetaRecipe> results = new HashSet<BetaRecipe>();
    rawAccumulateAllValues(POSITION_RECIPE, parameters, results);
    return results;
  }
  
  /**
   * Retrieve the set of values that occur in matches for recipe.
   * @return the Set of all values, null if no parameter with the given name exists, empty set if there are no matches
   * 
   */
  public Set<BetaRecipe> getAllValuesOfrecipe() {
    return rawAccumulateAllValuesOfrecipe(emptyArray());
  }
  
  /**
   * Retrieve the set of values that occur in matches for recipe.
   * @return the Set of all values, null if no parameter with the given name exists, empty set if there are no matches
   * 
   */
  public Set<BetaRecipe> getAllValuesOfrecipe(final BetaRecipeLeftParentMatch partialMatch) {
    return rawAccumulateAllValuesOfrecipe(partialMatch.toArray());
  }
  
  /**
   * Retrieve the set of values that occur in matches for recipe.
   * @return the Set of all values, null if no parameter with the given name exists, empty set if there are no matches
   * 
   */
  public Set<BetaRecipe> getAllValuesOfrecipe(final ProjectionIndexerRecipe pParent) {
    return rawAccumulateAllValuesOfrecipe(new Object[]{
    null, 
    pParent
    });
  }
  
  /**
   * Retrieve the set of values that occur in matches for parent.
   * @return the Set of all values, null if no parameter with the given name exists, empty set if there are no matches
   * 
   */
  protected Set<ProjectionIndexerRecipe> rawAccumulateAllValuesOfparent(final Object[] parameters) {
    Set<ProjectionIndexerRecipe> results = new HashSet<ProjectionIndexerRecipe>();
    rawAccumulateAllValues(POSITION_PARENT, parameters, results);
    return results;
  }
  
  /**
   * Retrieve the set of values that occur in matches for parent.
   * @return the Set of all values, null if no parameter with the given name exists, empty set if there are no matches
   * 
   */
  public Set<ProjectionIndexerRecipe> getAllValuesOfparent() {
    return rawAccumulateAllValuesOfparent(emptyArray());
  }
  
  /**
   * Retrieve the set of values that occur in matches for parent.
   * @return the Set of all values, null if no parameter with the given name exists, empty set if there are no matches
   * 
   */
  public Set<ProjectionIndexerRecipe> getAllValuesOfparent(final BetaRecipeLeftParentMatch partialMatch) {
    return rawAccumulateAllValuesOfparent(partialMatch.toArray());
  }
  
  /**
   * Retrieve the set of values that occur in matches for parent.
   * @return the Set of all values, null if no parameter with the given name exists, empty set if there are no matches
   * 
   */
  public Set<ProjectionIndexerRecipe> getAllValuesOfparent(final BetaRecipe pRecipe) {
    return rawAccumulateAllValuesOfparent(new Object[]{
    pRecipe, 
    null
    });
  }
  
  @Override
  protected BetaRecipeLeftParentMatch tupleToMatch(final Tuple t) {
    try {
    	return BetaRecipeLeftParentMatch.newMatch((BetaRecipe) t.get(POSITION_RECIPE), (ProjectionIndexerRecipe) t.get(POSITION_PARENT));
    } catch(ClassCastException e) {
    	LOGGER.error("Element(s) in tuple not properly typed!",e);
    	return null;
    }
  }
  
  @Override
  protected BetaRecipeLeftParentMatch arrayToMatch(final Object[] match) {
    try {
    	return BetaRecipeLeftParentMatch.newMatch((BetaRecipe) match[POSITION_RECIPE], (ProjectionIndexerRecipe) match[POSITION_PARENT]);
    } catch(ClassCastException e) {
    	LOGGER.error("Element(s) in array not properly typed!",e);
    	return null;
    }
  }
  
  @Override
  protected BetaRecipeLeftParentMatch arrayToMatchMutable(final Object[] match) {
    try {
    	return BetaRecipeLeftParentMatch.newMutableMatch((BetaRecipe) match[POSITION_RECIPE], (ProjectionIndexerRecipe) match[POSITION_PARENT]);
    } catch(ClassCastException e) {
    	LOGGER.error("Element(s) in array not properly typed!",e);
    	return null;
    }
  }
  
  /**
   * @return the singleton instance of the query specification of this pattern
   * @throws ViatraQueryException if the pattern definition could not be loaded
   * 
   */
  public static IQuerySpecification<BetaRecipeLeftParentMatcher> querySpecification() throws ViatraQueryException {
    return BetaRecipeLeftParentQuerySpecification.instance();
  }
}
