package org.eclipse.viatra.query.tooling.ui.retevis;

import java.util.Collection;
import java.util.HashSet;
import java.util.Set;
import org.apache.log4j.Logger;
import org.eclipse.emf.common.notify.Notifier;
import org.eclipse.viatra.query.runtime.api.IMatchProcessor;
import org.eclipse.viatra.query.runtime.api.IQuerySpecification;
import org.eclipse.viatra.query.runtime.api.ViatraQueryEngine;
import org.eclipse.viatra.query.runtime.api.impl.BaseMatcher;
import org.eclipse.viatra.query.runtime.exception.ViatraQueryException;
import org.eclipse.viatra.query.runtime.matchers.tuple.Tuple;
import org.eclipse.viatra.query.runtime.rete.recipes.InputFilterRecipe;
import org.eclipse.viatra.query.runtime.rete.recipes.Mask;
import org.eclipse.viatra.query.runtime.util.ViatraQueryLoggingUtil;
import org.eclipse.viatra.query.tooling.ui.retevis.InputFilterRecipeWithMaskMatch;
import org.eclipse.viatra.query.tooling.ui.retevis.util.InputFilterRecipeWithMaskQuerySpecification;

/**
 * Generated pattern matcher API of the org.eclipse.viatra.query.tooling.ui.retevis.inputFilterRecipeWithMask pattern,
 * providing pattern-specific query methods.
 * 
 * <p>Use the pattern matcher on a given model via {@link #on(ViatraQueryEngine)},
 * e.g. in conjunction with {@link ViatraQueryEngine#on(Notifier)}.
 * 
 * <p>Matches of the pattern will be represented as {@link InputFilterRecipeWithMaskMatch}.
 * 
 * <p>Original source:
 * <code><pre>
 * {@literal @}Item(item = recipe, label = "InputFilter: $recipe.inputKey$ {@literal @} $mask.sourceIndices$/$mask.sourceArity$")
 * {@literal @}Format(textColor = "#7F004D", color = "#CCCCCC")
 * {@literal @}QueryExplorer(checked = false)
 * pattern inputFilterRecipeWithMask(recipe : InputFilterRecipe, mask : Mask) {
 * 	InputFilterRecipe(recipe);
 * 	find inputFilterMask(recipe, mask);
 * }
 * </pre></code>
 * 
 * @see InputFilterRecipeWithMaskMatch
 * @see InputFilterRecipeWithMaskProcessor
 * @see InputFilterRecipeWithMaskQuerySpecification
 * 
 */
@SuppressWarnings("all")
public class InputFilterRecipeWithMaskMatcher extends BaseMatcher<InputFilterRecipeWithMaskMatch> {
  /**
   * Initializes the pattern matcher within an existing VIATRA Query engine.
   * If the pattern matcher is already constructed in the engine, only a light-weight reference is returned.
   * The match set will be incrementally refreshed upon updates.
   * @param engine the existing VIATRA Query engine in which this matcher will be created.
   * @throws ViatraQueryException if an error occurs during pattern matcher creation
   * 
   */
  public static InputFilterRecipeWithMaskMatcher on(final ViatraQueryEngine engine) throws ViatraQueryException {
    // check if matcher already exists
    InputFilterRecipeWithMaskMatcher matcher = engine.getExistingMatcher(querySpecification());
    if (matcher == null) {
    	matcher = new InputFilterRecipeWithMaskMatcher(engine);
    	// do not have to "put" it into engine.matchers, reportMatcherInitialized() will take care of it
    }
    return matcher;
  }
  
  private final static int POSITION_RECIPE = 0;
  
  private final static int POSITION_MASK = 1;
  
  private final static Logger LOGGER = ViatraQueryLoggingUtil.getLogger(InputFilterRecipeWithMaskMatcher.class);
  
  /**
   * Initializes the pattern matcher over a given EMF model root (recommended: Resource or ResourceSet).
   * If a pattern matcher is already constructed with the same root, only a light-weight reference is returned.
   * The scope of pattern matching will be the given EMF model root and below (see FAQ for more precise definition).
   * The match set will be incrementally refreshed upon updates from this scope.
   * <p>The matcher will be created within the managed {@link ViatraQueryEngine} belonging to the EMF model root, so
   * multiple matchers will reuse the same engine and benefit from increased performance and reduced memory footprint.
   * @param emfRoot the root of the EMF containment hierarchy where the pattern matcher will operate. Recommended: Resource or ResourceSet.
   * @throws ViatraQueryException if an error occurs during pattern matcher creation
   * @deprecated use {@link #on(ViatraQueryEngine)} instead, e.g. in conjunction with {@link ViatraQueryEngine#on(Notifier)}
   * 
   */
  @Deprecated
  public InputFilterRecipeWithMaskMatcher(final Notifier emfRoot) throws ViatraQueryException {
    this(ViatraQueryEngine.on(emfRoot));
  }
  
  /**
   * Initializes the pattern matcher within an existing VIATRA Query engine.
   * If the pattern matcher is already constructed in the engine, only a light-weight reference is returned.
   * The match set will be incrementally refreshed upon updates.
   * @param engine the existing VIATRA Query engine in which this matcher will be created.
   * @throws ViatraQueryException if an error occurs during pattern matcher creation
   * @deprecated use {@link #on(ViatraQueryEngine)} instead
   * 
   */
  @Deprecated
  public InputFilterRecipeWithMaskMatcher(final ViatraQueryEngine engine) throws ViatraQueryException {
    super(engine, querySpecification());
  }
  
  /**
   * Returns the set of all matches of the pattern that conform to the given fixed values of some parameters.
   * @param pRecipe the fixed value of pattern parameter recipe, or null if not bound.
   * @param pMask the fixed value of pattern parameter mask, or null if not bound.
   * @return matches represented as a InputFilterRecipeWithMaskMatch object.
   * 
   */
  public Collection<InputFilterRecipeWithMaskMatch> getAllMatches(final InputFilterRecipe pRecipe, final Mask pMask) {
    return rawGetAllMatches(new Object[]{pRecipe, pMask});
  }
  
  /**
   * Returns an arbitrarily chosen match of the pattern that conforms to the given fixed values of some parameters.
   * Neither determinism nor randomness of selection is guaranteed.
   * @param pRecipe the fixed value of pattern parameter recipe, or null if not bound.
   * @param pMask the fixed value of pattern parameter mask, or null if not bound.
   * @return a match represented as a InputFilterRecipeWithMaskMatch object, or null if no match is found.
   * 
   */
  public InputFilterRecipeWithMaskMatch getOneArbitraryMatch(final InputFilterRecipe pRecipe, final Mask pMask) {
    return rawGetOneArbitraryMatch(new Object[]{pRecipe, pMask});
  }
  
  /**
   * Indicates whether the given combination of specified pattern parameters constitute a valid pattern match,
   * under any possible substitution of the unspecified parameters (if any).
   * @param pRecipe the fixed value of pattern parameter recipe, or null if not bound.
   * @param pMask the fixed value of pattern parameter mask, or null if not bound.
   * @return true if the input is a valid (partial) match of the pattern.
   * 
   */
  public boolean hasMatch(final InputFilterRecipe pRecipe, final Mask pMask) {
    return rawHasMatch(new Object[]{pRecipe, pMask});
  }
  
  /**
   * Returns the number of all matches of the pattern that conform to the given fixed values of some parameters.
   * @param pRecipe the fixed value of pattern parameter recipe, or null if not bound.
   * @param pMask the fixed value of pattern parameter mask, or null if not bound.
   * @return the number of pattern matches found.
   * 
   */
  public int countMatches(final InputFilterRecipe pRecipe, final Mask pMask) {
    return rawCountMatches(new Object[]{pRecipe, pMask});
  }
  
  /**
   * Executes the given processor on each match of the pattern that conforms to the given fixed values of some parameters.
   * @param pRecipe the fixed value of pattern parameter recipe, or null if not bound.
   * @param pMask the fixed value of pattern parameter mask, or null if not bound.
   * @param processor the action that will process each pattern match.
   * 
   */
  public void forEachMatch(final InputFilterRecipe pRecipe, final Mask pMask, final IMatchProcessor<? super InputFilterRecipeWithMaskMatch> processor) {
    rawForEachMatch(new Object[]{pRecipe, pMask}, processor);
  }
  
  /**
   * Executes the given processor on an arbitrarily chosen match of the pattern that conforms to the given fixed values of some parameters.
   * Neither determinism nor randomness of selection is guaranteed.
   * @param pRecipe the fixed value of pattern parameter recipe, or null if not bound.
   * @param pMask the fixed value of pattern parameter mask, or null if not bound.
   * @param processor the action that will process the selected match.
   * @return true if the pattern has at least one match with the given parameter values, false if the processor was not invoked
   * 
   */
  public boolean forOneArbitraryMatch(final InputFilterRecipe pRecipe, final Mask pMask, final IMatchProcessor<? super InputFilterRecipeWithMaskMatch> processor) {
    return rawForOneArbitraryMatch(new Object[]{pRecipe, pMask}, processor);
  }
  
  /**
   * Returns a new (partial) match.
   * This can be used e.g. to call the matcher with a partial match.
   * <p>The returned match will be immutable. Use {@link #newEmptyMatch()} to obtain a mutable match object.
   * @param pRecipe the fixed value of pattern parameter recipe, or null if not bound.
   * @param pMask the fixed value of pattern parameter mask, or null if not bound.
   * @return the (partial) match object.
   * 
   */
  public InputFilterRecipeWithMaskMatch newMatch(final InputFilterRecipe pRecipe, final Mask pMask) {
    return InputFilterRecipeWithMaskMatch.newMatch(pRecipe, pMask);
  }
  
  /**
   * Retrieve the set of values that occur in matches for recipe.
   * @return the Set of all values, null if no parameter with the given name exists, empty set if there are no matches
   * 
   */
  protected Set<InputFilterRecipe> rawAccumulateAllValuesOfrecipe(final Object[] parameters) {
    Set<InputFilterRecipe> results = new HashSet<InputFilterRecipe>();
    rawAccumulateAllValues(POSITION_RECIPE, parameters, results);
    return results;
  }
  
  /**
   * Retrieve the set of values that occur in matches for recipe.
   * @return the Set of all values, null if no parameter with the given name exists, empty set if there are no matches
   * 
   */
  public Set<InputFilterRecipe> getAllValuesOfrecipe() {
    return rawAccumulateAllValuesOfrecipe(emptyArray());
  }
  
  /**
   * Retrieve the set of values that occur in matches for recipe.
   * @return the Set of all values, null if no parameter with the given name exists, empty set if there are no matches
   * 
   */
  public Set<InputFilterRecipe> getAllValuesOfrecipe(final InputFilterRecipeWithMaskMatch partialMatch) {
    return rawAccumulateAllValuesOfrecipe(partialMatch.toArray());
  }
  
  /**
   * Retrieve the set of values that occur in matches for recipe.
   * @return the Set of all values, null if no parameter with the given name exists, empty set if there are no matches
   * 
   */
  public Set<InputFilterRecipe> getAllValuesOfrecipe(final Mask pMask) {
    return rawAccumulateAllValuesOfrecipe(new Object[]{
    null, 
    pMask
    });
  }
  
  /**
   * Retrieve the set of values that occur in matches for mask.
   * @return the Set of all values, null if no parameter with the given name exists, empty set if there are no matches
   * 
   */
  protected Set<Mask> rawAccumulateAllValuesOfmask(final Object[] parameters) {
    Set<Mask> results = new HashSet<Mask>();
    rawAccumulateAllValues(POSITION_MASK, parameters, results);
    return results;
  }
  
  /**
   * Retrieve the set of values that occur in matches for mask.
   * @return the Set of all values, null if no parameter with the given name exists, empty set if there are no matches
   * 
   */
  public Set<Mask> getAllValuesOfmask() {
    return rawAccumulateAllValuesOfmask(emptyArray());
  }
  
  /**
   * Retrieve the set of values that occur in matches for mask.
   * @return the Set of all values, null if no parameter with the given name exists, empty set if there are no matches
   * 
   */
  public Set<Mask> getAllValuesOfmask(final InputFilterRecipeWithMaskMatch partialMatch) {
    return rawAccumulateAllValuesOfmask(partialMatch.toArray());
  }
  
  /**
   * Retrieve the set of values that occur in matches for mask.
   * @return the Set of all values, null if no parameter with the given name exists, empty set if there are no matches
   * 
   */
  public Set<Mask> getAllValuesOfmask(final InputFilterRecipe pRecipe) {
    return rawAccumulateAllValuesOfmask(new Object[]{
    pRecipe, 
    null
    });
  }
  
  @Override
  protected InputFilterRecipeWithMaskMatch tupleToMatch(final Tuple t) {
    try {
    	return InputFilterRecipeWithMaskMatch.newMatch((InputFilterRecipe) t.get(POSITION_RECIPE), (Mask) t.get(POSITION_MASK));
    } catch(ClassCastException e) {
    	LOGGER.error("Element(s) in tuple not properly typed!",e);
    	return null;
    }
  }
  
  @Override
  protected InputFilterRecipeWithMaskMatch arrayToMatch(final Object[] match) {
    try {
    	return InputFilterRecipeWithMaskMatch.newMatch((InputFilterRecipe) match[POSITION_RECIPE], (Mask) match[POSITION_MASK]);
    } catch(ClassCastException e) {
    	LOGGER.error("Element(s) in array not properly typed!",e);
    	return null;
    }
  }
  
  @Override
  protected InputFilterRecipeWithMaskMatch arrayToMatchMutable(final Object[] match) {
    try {
    	return InputFilterRecipeWithMaskMatch.newMutableMatch((InputFilterRecipe) match[POSITION_RECIPE], (Mask) match[POSITION_MASK]);
    } catch(ClassCastException e) {
    	LOGGER.error("Element(s) in array not properly typed!",e);
    	return null;
    }
  }
  
  /**
   * @return the singleton instance of the query specification of this pattern
   * @throws ViatraQueryException if the pattern definition could not be loaded
   * 
   */
  public static IQuerySpecification<InputFilterRecipeWithMaskMatcher> querySpecification() throws ViatraQueryException {
    return InputFilterRecipeWithMaskQuerySpecification.instance();
  }
}
