package org.eclipse.viatra.query.tooling.ui.retevis.util;

import com.google.common.collect.Sets;
import java.util.Arrays;
import java.util.List;
import java.util.Set;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.viatra.query.runtime.api.ViatraQueryEngine;
import org.eclipse.viatra.query.runtime.api.impl.BaseGeneratedEMFPQuery;
import org.eclipse.viatra.query.runtime.api.impl.BaseGeneratedEMFQuerySpecification;
import org.eclipse.viatra.query.runtime.emf.types.EClassTransitiveInstancesKey;
import org.eclipse.viatra.query.runtime.exception.ViatraQueryException;
import org.eclipse.viatra.query.runtime.matchers.psystem.PBody;
import org.eclipse.viatra.query.runtime.matchers.psystem.PVariable;
import org.eclipse.viatra.query.runtime.matchers.psystem.annotations.PAnnotation;
import org.eclipse.viatra.query.runtime.matchers.psystem.annotations.ParameterReference;
import org.eclipse.viatra.query.runtime.matchers.psystem.basicdeferred.ExportedParameter;
import org.eclipse.viatra.query.runtime.matchers.psystem.basicenumerables.TypeConstraint;
import org.eclipse.viatra.query.runtime.matchers.psystem.queries.PParameter;
import org.eclipse.viatra.query.runtime.matchers.psystem.queries.QueryInitializationException;
import org.eclipse.viatra.query.runtime.matchers.tuple.FlatTuple;
import org.eclipse.viatra.query.tooling.ui.retevis.CheckRecipeMatch;
import org.eclipse.viatra.query.tooling.ui.retevis.CheckRecipeMatcher;

/**
 * A pattern-specific query specification that can instantiate CheckRecipeMatcher in a type-safe way.
 * 
 * @see CheckRecipeMatcher
 * @see CheckRecipeMatch
 * 
 */
@SuppressWarnings("all")
public final class CheckRecipeQuerySpecification extends BaseGeneratedEMFQuerySpecification<CheckRecipeMatcher> {
  private CheckRecipeQuerySpecification() {
    super(GeneratedPQuery.INSTANCE);
  }
  
  /**
   * @return the singleton instance of the query specification
   * @throws ViatraQueryException if the pattern definition could not be loaded
   * 
   */
  public static CheckRecipeQuerySpecification instance() throws ViatraQueryException {
    try{
    	return LazyHolder.INSTANCE;
    } catch (ExceptionInInitializerError err) {
    	throw processInitializerError(err);
    }
  }
  
  @Override
  protected CheckRecipeMatcher instantiate(final ViatraQueryEngine engine) throws ViatraQueryException {
    return CheckRecipeMatcher.on(engine);
  }
  
  @Override
  public CheckRecipeMatch newEmptyMatch() {
    return CheckRecipeMatch.newEmptyMatch();
  }
  
  @Override
  public CheckRecipeMatch newMatch(final Object... parameters) {
    return CheckRecipeMatch.newMatch((org.eclipse.viatra.query.runtime.rete.recipes.CheckRecipe) parameters[0]);
  }
  
  /**
   * Inner class allowing the singleton instance of {@link CheckRecipeQuerySpecification} to be created 
   * 	<b>not</b> at the class load time of the outer class, 
   * 	but rather at the first call to {@link CheckRecipeQuerySpecification#instance()}.
   * 
   * <p> This workaround is required e.g. to support recursion.
   * 
   */
  private static class LazyHolder {
    private final static CheckRecipeQuerySpecification INSTANCE = new CheckRecipeQuerySpecification();
    
    /**
     * Statically initializes the query specification <b>after</b> the field {@link #INSTANCE} is assigned.
     * This initialization order is required to support indirect recursion.
     * 
     * <p> The static initializer is defined using a helper field to work around limitations of the code generator.
     * 
     */
    private final static Object STATIC_INITIALIZER = ensureInitialized();
    
    public static Object ensureInitialized() {
      INSTANCE.ensureInitializedInternalSneaky();
      return null;					
    }
  }
  
  private static class GeneratedPQuery extends BaseGeneratedEMFPQuery {
    private final static CheckRecipeQuerySpecification.GeneratedPQuery INSTANCE = new GeneratedPQuery();
    
    @Override
    public String getFullyQualifiedName() {
      return "org.eclipse.viatra.query.tooling.ui.retevis.checkRecipe";
    }
    
    @Override
    public List<String> getParameterNames() {
      return Arrays.asList("recipe");
    }
    
    @Override
    public List<PParameter> getParameters() {
      return Arrays.asList(new PParameter("recipe", "org.eclipse.viatra.query.runtime.rete.recipes.CheckRecipe"));
    }
    
    @Override
    public Set<PBody> doGetContainedBodies() throws QueryInitializationException {
      Set<PBody> bodies = Sets.newLinkedHashSet();
      try {
      	{
      		PBody body = new PBody(this);
      		PVariable var_recipe = body.getOrCreateVariableByName("recipe");
      		new TypeConstraint(body, new FlatTuple(var_recipe), new EClassTransitiveInstancesKey((EClass)getClassifierLiteral("http://www.eclipse.org/viatra/query/rete/recipes", "CheckRecipe")));
      		body.setSymbolicParameters(Arrays.<ExportedParameter>asList(
      		   new ExportedParameter(body, var_recipe, "recipe")
      		));
      		// 	CheckRecipe(recipe)
      		new TypeConstraint(body, new FlatTuple(var_recipe), new EClassTransitiveInstancesKey((EClass)getClassifierLiteral("http://www.eclipse.org/viatra/query/rete/recipes", "CheckRecipe")));
      		bodies.add(body);
      	}
      	                {
      		PAnnotation annotation = new PAnnotation("Item");
      		annotation.addAttribute("item", new ParameterReference("recipe"));
      		annotation.addAttribute("label", "Check");
      		addAnnotation(annotation);
      	}
      	                {
      		PAnnotation annotation = new PAnnotation("QueryExplorer");
      		annotation.addAttribute("checked", false);
      		addAnnotation(annotation);
      	}
      	// to silence compiler error
      	if (false) throw new ViatraQueryException("Never", "happens");
      } catch (ViatraQueryException ex) {
      	throw processDependencyException(ex);
      }
      return bodies;
    }
  }
}
