package org.eclipse.viatra.query.tooling.ui.retevis.util;

import com.google.common.collect.Sets;
import java.util.Arrays;
import java.util.List;
import java.util.Set;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.viatra.query.runtime.api.ViatraQueryEngine;
import org.eclipse.viatra.query.runtime.api.impl.BaseGeneratedEMFPQuery;
import org.eclipse.viatra.query.runtime.api.impl.BaseGeneratedEMFQuerySpecification;
import org.eclipse.viatra.query.runtime.emf.types.EClassTransitiveInstancesKey;
import org.eclipse.viatra.query.runtime.emf.types.EStructuralFeatureInstancesKey;
import org.eclipse.viatra.query.runtime.exception.ViatraQueryException;
import org.eclipse.viatra.query.runtime.matchers.psystem.PBody;
import org.eclipse.viatra.query.runtime.matchers.psystem.PVariable;
import org.eclipse.viatra.query.runtime.matchers.psystem.annotations.PAnnotation;
import org.eclipse.viatra.query.runtime.matchers.psystem.annotations.ParameterReference;
import org.eclipse.viatra.query.runtime.matchers.psystem.basicdeferred.Equality;
import org.eclipse.viatra.query.runtime.matchers.psystem.basicdeferred.ExportedParameter;
import org.eclipse.viatra.query.runtime.matchers.psystem.basicenumerables.TypeConstraint;
import org.eclipse.viatra.query.runtime.matchers.psystem.queries.PParameter;
import org.eclipse.viatra.query.runtime.matchers.psystem.queries.QueryInitializationException;
import org.eclipse.viatra.query.runtime.matchers.tuple.FlatTuple;
import org.eclipse.viatra.query.tooling.ui.retevis.JoinRecipeMatch;
import org.eclipse.viatra.query.tooling.ui.retevis.JoinRecipeMatcher;

/**
 * A pattern-specific query specification that can instantiate JoinRecipeMatcher in a type-safe way.
 * 
 * @see JoinRecipeMatcher
 * @see JoinRecipeMatch
 * 
 */
@SuppressWarnings("all")
public final class JoinRecipeQuerySpecification extends BaseGeneratedEMFQuerySpecification<JoinRecipeMatcher> {
  private JoinRecipeQuerySpecification() {
    super(GeneratedPQuery.INSTANCE);
  }
  
  /**
   * @return the singleton instance of the query specification
   * @throws ViatraQueryException if the pattern definition could not be loaded
   * 
   */
  public static JoinRecipeQuerySpecification instance() throws ViatraQueryException {
    try{
    	return LazyHolder.INSTANCE;
    } catch (ExceptionInInitializerError err) {
    	throw processInitializerError(err);
    }
  }
  
  @Override
  protected JoinRecipeMatcher instantiate(final ViatraQueryEngine engine) throws ViatraQueryException {
    return JoinRecipeMatcher.on(engine);
  }
  
  @Override
  public JoinRecipeMatch newEmptyMatch() {
    return JoinRecipeMatch.newEmptyMatch();
  }
  
  @Override
  public JoinRecipeMatch newMatch(final Object... parameters) {
    return JoinRecipeMatch.newMatch((org.eclipse.viatra.query.runtime.rete.recipes.JoinRecipe) parameters[0], (org.eclipse.viatra.query.runtime.rete.recipes.Mask) parameters[1]);
  }
  
  /**
   * Inner class allowing the singleton instance of {@link JoinRecipeQuerySpecification} to be created 
   * 	<b>not</b> at the class load time of the outer class, 
   * 	but rather at the first call to {@link JoinRecipeQuerySpecification#instance()}.
   * 
   * <p> This workaround is required e.g. to support recursion.
   * 
   */
  private static class LazyHolder {
    private final static JoinRecipeQuerySpecification INSTANCE = new JoinRecipeQuerySpecification();
    
    /**
     * Statically initializes the query specification <b>after</b> the field {@link #INSTANCE} is assigned.
     * This initialization order is required to support indirect recursion.
     * 
     * <p> The static initializer is defined using a helper field to work around limitations of the code generator.
     * 
     */
    private final static Object STATIC_INITIALIZER = ensureInitialized();
    
    public static Object ensureInitialized() {
      INSTANCE.ensureInitializedInternalSneaky();
      return null;					
    }
  }
  
  private static class GeneratedPQuery extends BaseGeneratedEMFPQuery {
    private final static JoinRecipeQuerySpecification.GeneratedPQuery INSTANCE = new GeneratedPQuery();
    
    @Override
    public String getFullyQualifiedName() {
      return "org.eclipse.viatra.query.tooling.ui.retevis.joinRecipe";
    }
    
    @Override
    public List<String> getParameterNames() {
      return Arrays.asList("recipe","rightParentComplementaryMask");
    }
    
    @Override
    public List<PParameter> getParameters() {
      return Arrays.asList(new PParameter("recipe", "org.eclipse.viatra.query.runtime.rete.recipes.JoinRecipe"),new PParameter("rightParentComplementaryMask", "org.eclipse.viatra.query.runtime.rete.recipes.Mask"));
    }
    
    @Override
    public Set<PBody> doGetContainedBodies() throws QueryInitializationException {
      Set<PBody> bodies = Sets.newLinkedHashSet();
      try {
      	{
      		PBody body = new PBody(this);
      		PVariable var_recipe = body.getOrCreateVariableByName("recipe");
      		PVariable var_rightParentComplementaryMask = body.getOrCreateVariableByName("rightParentComplementaryMask");
      		new TypeConstraint(body, new FlatTuple(var_recipe), new EClassTransitiveInstancesKey((EClass)getClassifierLiteral("http://www.eclipse.org/viatra/query/rete/recipes", "JoinRecipe")));
      		new TypeConstraint(body, new FlatTuple(var_rightParentComplementaryMask), new EClassTransitiveInstancesKey((EClass)getClassifierLiteral("http://www.eclipse.org/viatra/query/rete/recipes", "Mask")));
      		body.setSymbolicParameters(Arrays.<ExportedParameter>asList(
      		   new ExportedParameter(body, var_recipe, "recipe"),
      		   new ExportedParameter(body, var_rightParentComplementaryMask, "rightParentComplementaryMask")
      		));
      		// 	JoinRecipe(recipe)
      		new TypeConstraint(body, new FlatTuple(var_recipe), new EClassTransitiveInstancesKey((EClass)getClassifierLiteral("http://www.eclipse.org/viatra/query/rete/recipes", "JoinRecipe")));
      		// 	JoinRecipe.rightParentComplementaryMask(recipe, rightParentComplementaryMask)
      		new TypeConstraint(body, new FlatTuple(var_recipe), new EClassTransitiveInstancesKey((EClass)getClassifierLiteral("http://www.eclipse.org/viatra/query/rete/recipes", "JoinRecipe")));
      		PVariable var__virtual_0_ = body.getOrCreateVariableByName(".virtual{0}");
      		new TypeConstraint(body, new FlatTuple(var_recipe, var__virtual_0_), new EStructuralFeatureInstancesKey(getFeatureLiteral("http://www.eclipse.org/viatra/query/rete/recipes", "JoinRecipe", "rightParentComplementaryMask")));
      		new Equality(body, var__virtual_0_, var_rightParentComplementaryMask);
      		bodies.add(body);
      	}
      	                {
      		PAnnotation annotation = new PAnnotation("Item");
      		annotation.addAttribute("item", new ParameterReference("recipe"));
      		annotation.addAttribute("label", "Join left + $rightParentComplementaryMask.sourceIndices$/$rightParentComplementaryMask.sourceArity$");
      		addAnnotation(annotation);
      	}
      	                {
      		PAnnotation annotation = new PAnnotation("QueryExplorer");
      		annotation.addAttribute("checked", false);
      		addAnnotation(annotation);
      	}
      	// to silence compiler error
      	if (false) throw new ViatraQueryException("Never", "happens");
      } catch (ViatraQueryException ex) {
      	throw processDependencyException(ex);
      }
      return bodies;
    }
  }
}
