package org.eclipse.viatra.query.tooling.ui.retevis.util;

import com.google.common.collect.Sets;
import java.util.Arrays;
import java.util.List;
import java.util.Set;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.viatra.query.runtime.api.ViatraQueryEngine;
import org.eclipse.viatra.query.runtime.api.impl.BaseGeneratedEMFPQuery;
import org.eclipse.viatra.query.runtime.api.impl.BaseGeneratedEMFQuerySpecification;
import org.eclipse.viatra.query.runtime.emf.types.EClassTransitiveInstancesKey;
import org.eclipse.viatra.query.runtime.emf.types.EStructuralFeatureInstancesKey;
import org.eclipse.viatra.query.runtime.exception.ViatraQueryException;
import org.eclipse.viatra.query.runtime.matchers.psystem.PBody;
import org.eclipse.viatra.query.runtime.matchers.psystem.PVariable;
import org.eclipse.viatra.query.runtime.matchers.psystem.annotations.PAnnotation;
import org.eclipse.viatra.query.runtime.matchers.psystem.annotations.ParameterReference;
import org.eclipse.viatra.query.runtime.matchers.psystem.basicdeferred.Equality;
import org.eclipse.viatra.query.runtime.matchers.psystem.basicdeferred.ExportedParameter;
import org.eclipse.viatra.query.runtime.matchers.psystem.basicenumerables.TypeConstraint;
import org.eclipse.viatra.query.runtime.matchers.psystem.queries.PParameter;
import org.eclipse.viatra.query.runtime.matchers.psystem.queries.QueryInitializationException;
import org.eclipse.viatra.query.runtime.matchers.tuple.FlatTuple;
import org.eclipse.viatra.query.tooling.ui.retevis.ProjectionIndexerRecipeMatch;
import org.eclipse.viatra.query.tooling.ui.retevis.ProjectionIndexerRecipeMatcher;

/**
 * A pattern-specific query specification that can instantiate ProjectionIndexerRecipeMatcher in a type-safe way.
 * 
 * @see ProjectionIndexerRecipeMatcher
 * @see ProjectionIndexerRecipeMatch
 * 
 */
@SuppressWarnings("all")
public final class ProjectionIndexerRecipeQuerySpecification extends BaseGeneratedEMFQuerySpecification<ProjectionIndexerRecipeMatcher> {
  private ProjectionIndexerRecipeQuerySpecification() {
    super(GeneratedPQuery.INSTANCE);
  }
  
  /**
   * @return the singleton instance of the query specification
   * @throws ViatraQueryException if the pattern definition could not be loaded
   * 
   */
  public static ProjectionIndexerRecipeQuerySpecification instance() throws ViatraQueryException {
    try{
    	return LazyHolder.INSTANCE;
    } catch (ExceptionInInitializerError err) {
    	throw processInitializerError(err);
    }
  }
  
  @Override
  protected ProjectionIndexerRecipeMatcher instantiate(final ViatraQueryEngine engine) throws ViatraQueryException {
    return ProjectionIndexerRecipeMatcher.on(engine);
  }
  
  @Override
  public ProjectionIndexerRecipeMatch newEmptyMatch() {
    return ProjectionIndexerRecipeMatch.newEmptyMatch();
  }
  
  @Override
  public ProjectionIndexerRecipeMatch newMatch(final Object... parameters) {
    return ProjectionIndexerRecipeMatch.newMatch((org.eclipse.viatra.query.runtime.rete.recipes.ProjectionIndexerRecipe) parameters[0], (org.eclipse.viatra.query.runtime.rete.recipes.Mask) parameters[1]);
  }
  
  /**
   * Inner class allowing the singleton instance of {@link ProjectionIndexerRecipeQuerySpecification} to be created 
   * 	<b>not</b> at the class load time of the outer class, 
   * 	but rather at the first call to {@link ProjectionIndexerRecipeQuerySpecification#instance()}.
   * 
   * <p> This workaround is required e.g. to support recursion.
   * 
   */
  private static class LazyHolder {
    private final static ProjectionIndexerRecipeQuerySpecification INSTANCE = new ProjectionIndexerRecipeQuerySpecification();
    
    /**
     * Statically initializes the query specification <b>after</b> the field {@link #INSTANCE} is assigned.
     * This initialization order is required to support indirect recursion.
     * 
     * <p> The static initializer is defined using a helper field to work around limitations of the code generator.
     * 
     */
    private final static Object STATIC_INITIALIZER = ensureInitialized();
    
    public static Object ensureInitialized() {
      INSTANCE.ensureInitializedInternalSneaky();
      return null;					
    }
  }
  
  private static class GeneratedPQuery extends BaseGeneratedEMFPQuery {
    private final static ProjectionIndexerRecipeQuerySpecification.GeneratedPQuery INSTANCE = new GeneratedPQuery();
    
    @Override
    public String getFullyQualifiedName() {
      return "org.eclipse.viatra.query.tooling.ui.retevis.projectionIndexerRecipe";
    }
    
    @Override
    public List<String> getParameterNames() {
      return Arrays.asList("recipe","mask");
    }
    
    @Override
    public List<PParameter> getParameters() {
      return Arrays.asList(new PParameter("recipe", "org.eclipse.viatra.query.runtime.rete.recipes.ProjectionIndexerRecipe"),new PParameter("mask", "org.eclipse.viatra.query.runtime.rete.recipes.Mask"));
    }
    
    @Override
    public Set<PBody> doGetContainedBodies() throws QueryInitializationException {
      Set<PBody> bodies = Sets.newLinkedHashSet();
      try {
      	{
      		PBody body = new PBody(this);
      		PVariable var_recipe = body.getOrCreateVariableByName("recipe");
      		PVariable var_mask = body.getOrCreateVariableByName("mask");
      		new TypeConstraint(body, new FlatTuple(var_recipe), new EClassTransitiveInstancesKey((EClass)getClassifierLiteral("http://www.eclipse.org/viatra/query/rete/recipes", "ProjectionIndexerRecipe")));
      		new TypeConstraint(body, new FlatTuple(var_mask), new EClassTransitiveInstancesKey((EClass)getClassifierLiteral("http://www.eclipse.org/viatra/query/rete/recipes", "Mask")));
      		body.setSymbolicParameters(Arrays.<ExportedParameter>asList(
      		   new ExportedParameter(body, var_recipe, "recipe"),
      		   new ExportedParameter(body, var_mask, "mask")
      		));
      		// 	ProjectionIndexerRecipe(recipe)
      		new TypeConstraint(body, new FlatTuple(var_recipe), new EClassTransitiveInstancesKey((EClass)getClassifierLiteral("http://www.eclipse.org/viatra/query/rete/recipes", "ProjectionIndexerRecipe")));
      		// 	ProjectionIndexerRecipe.mask(recipe, mask)
      		new TypeConstraint(body, new FlatTuple(var_recipe), new EClassTransitiveInstancesKey((EClass)getClassifierLiteral("http://www.eclipse.org/viatra/query/rete/recipes", "ProjectionIndexerRecipe")));
      		PVariable var__virtual_0_ = body.getOrCreateVariableByName(".virtual{0}");
      		new TypeConstraint(body, new FlatTuple(var_recipe, var__virtual_0_), new EStructuralFeatureInstancesKey(getFeatureLiteral("http://www.eclipse.org/viatra/query/rete/recipes", "IndexerRecipe", "mask")));
      		new Equality(body, var__virtual_0_, var_mask);
      		bodies.add(body);
      	}
      	                {
      		PAnnotation annotation = new PAnnotation("Item");
      		annotation.addAttribute("item", new ParameterReference("recipe"));
      		annotation.addAttribute("label", "IX: $mask.sourceIndices$/$mask.sourceArity$");
      		addAnnotation(annotation);
      	}
      	                {
      		PAnnotation annotation = new PAnnotation("Format");
      		annotation.addAttribute("textColor", "#000000");
      		annotation.addAttribute("color", "#FFFFFF");
      		addAnnotation(annotation);
      	}
      	                {
      		PAnnotation annotation = new PAnnotation("QueryExplorer");
      		annotation.addAttribute("checked", false);
      		addAnnotation(annotation);
      	}
      	// to silence compiler error
      	if (false) throw new ViatraQueryException("Never", "happens");
      } catch (ViatraQueryException ex) {
      	throw processDependencyException(ex);
      }
      return bodies;
    }
  }
}
