/**
 * Copyright (c) 2004-2015, Peter Lunk, Zoltan Ujhelyi and Daniel Varro
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *   Peter Lunk - initial API and implementation
 */
package org.eclipse.viatra.transformation.debug.adapter.impl;

import com.google.common.base.Objects;
import com.google.common.base.Preconditions;
import com.google.common.collect.Lists;
import java.util.ArrayList;
import java.util.List;
import org.eclipse.viatra.query.runtime.api.ViatraQueryEngine;
import org.eclipse.viatra.transformation.debug.adapter.IAdapterConfiguration;
import org.eclipse.viatra.transformation.debug.adapter.ITransformationAdapter;
import org.eclipse.viatra.transformation.debug.adapter.impl.AdaptableExecutor;
import org.eclipse.viatra.transformation.evm.api.Context;
import org.eclipse.viatra.transformation.evm.api.Executor;
import org.eclipse.viatra.transformation.evm.api.event.EventRealm;
import org.eclipse.viatra.transformation.evm.specific.event.ViatraQueryEventRealm;

/**
 * Builder class that is responsible for creating an adapter supporting executor.
 * 
 * @author Peter Lunk
 */
@SuppressWarnings("all")
public class AdaptableExecutorBuilder {
  protected ViatraQueryEngine engine;
  
  protected EventRealm eventRealm;
  
  protected List<ITransformationAdapter> adapters;
  
  protected Context context;
  
  public AdaptableExecutorBuilder() {
    ArrayList<ITransformationAdapter> _newArrayList = Lists.<ITransformationAdapter>newArrayList();
    this.adapters = _newArrayList;
  }
  
  public AdaptableExecutorBuilder setQueryEngine(final ViatraQueryEngine engine) {
    AdaptableExecutorBuilder _xblockexpression = null;
    {
      this.engine = engine;
      _xblockexpression = this;
    }
    return _xblockexpression;
  }
  
  public AdaptableExecutorBuilder setEventRealm(final EventRealm evetRealm) {
    AdaptableExecutorBuilder _xblockexpression = null;
    {
      this.eventRealm = evetRealm;
      _xblockexpression = this;
    }
    return _xblockexpression;
  }
  
  public AdaptableExecutorBuilder setContext(final Context context) {
    AdaptableExecutorBuilder _xblockexpression = null;
    {
      this.context = context;
      _xblockexpression = this;
    }
    return _xblockexpression;
  }
  
  public AdaptableExecutorBuilder addAdapter(final ITransformationAdapter adapter) {
    AdaptableExecutorBuilder _xblockexpression = null;
    {
      this.adapters.add(adapter);
      _xblockexpression = this;
    }
    return _xblockexpression;
  }
  
  public AdaptableExecutorBuilder addConfiguration(final IAdapterConfiguration configuration) {
    AdaptableExecutorBuilder _xblockexpression = null;
    {
      List<ITransformationAdapter> _adapters = configuration.getAdapters();
      this.adapters.addAll(_adapters);
      _xblockexpression = this;
    }
    return _xblockexpression;
  }
  
  public Executor build() {
    AdaptableExecutor _xblockexpression = null;
    {
      boolean _notEquals = (!Objects.equal(this.engine, null));
      Preconditions.checkArgument(_notEquals, "Engine cannot be null");
      boolean _equals = Objects.equal(this.eventRealm, null);
      if (_equals) {
        ViatraQueryEventRealm _create = ViatraQueryEventRealm.create(this.engine);
        this.eventRealm = _create;
      }
      boolean _equals_1 = Objects.equal(this.context, null);
      if (_equals_1) {
        Context _create_1 = Context.create();
        this.context = _create_1;
      }
      _xblockexpression = new AdaptableExecutor(this.eventRealm, this.context, this.adapters);
    }
    return _xblockexpression;
  }
}
