/**
 * Copyright (c) 2004-2015, Peter Lunk, Zoltan Ujhelyi and Daniel Varro
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *   Peter Lunk - initial API and implementation
 */
package org.eclipse.viatra.transformation.debug.configuration;

import com.google.common.collect.Lists;
import java.util.ArrayList;
import java.util.List;
import org.eclipse.viatra.transformation.debug.TransformationDebugger;
import org.eclipse.viatra.transformation.debug.adapter.IAdapterConfiguration;
import org.eclipse.viatra.transformation.debug.adapter.ITransformationAdapter;
import org.eclipse.viatra.transformation.debug.breakpoints.ITransformationBreakpoint;
import org.eclipse.viatra.transformation.debug.controller.IDebugController;
import org.eclipse.viatra.transformation.debug.controller.impl.ConsoleDebugger;

/**
 * Configuration class that defines the debugger.
 * 
 * @author Peter Lunk
 */
@SuppressWarnings("all")
public class TransformationDebuggerConfiguration implements IAdapterConfiguration {
  private List<ITransformationAdapter> adapters;
  
  public TransformationDebuggerConfiguration(final IDebugController usedController, final ITransformationBreakpoint... breakpoints) {
    ArrayList<ITransformationAdapter> _newArrayList = Lists.<ITransformationAdapter>newArrayList();
    this.adapters = _newArrayList;
    TransformationDebugger _transformationDebugger = new TransformationDebugger(usedController, breakpoints);
    this.adapters.add(_transformationDebugger);
  }
  
  public TransformationDebuggerConfiguration(final ITransformationBreakpoint... breakpoints) {
    final ConsoleDebugger usedUI = new ConsoleDebugger();
    ArrayList<ITransformationAdapter> _newArrayList = Lists.<ITransformationAdapter>newArrayList();
    this.adapters = _newArrayList;
    TransformationDebugger _transformationDebugger = new TransformationDebugger(usedUI, breakpoints);
    this.adapters.add(_transformationDebugger);
  }
  
  @Override
  public List<ITransformationAdapter> getAdapters() {
    return this.adapters;
  }
}
